unit frmmain;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, LResources, Forms, Controls, Graphics, Dialogs, ComCtrls,
  Menus, ActnList, ExtCtrls, fpjson, grids , StdCtrls, jsonparser;

type

  { TMainForm }

  TMainForm = class(TForm)
    AGoto: TAction;
    ANewElement: TAction;
    AQuit: TAction;
    ASaveAs: TAction;
    ASave: TAction;
    AOpen: TAction;
    ANew: TAction;
    ALMain: TActionList;
    ILMain: TImageList;
    MainMenu1: TMainMenu;
    Edit: TMenuItem;
    MIGoto: TMenuItem;
    MINewNode: TMenuItem;
    MISep1: TMenuItem;
    MIQuit: TMenuItem;
    MISaveAs: TMenuItem;
    MISave: TMenuItem;
    MIOpen: TMenuItem;
    MINew: TMenuItem;
    MFiLe: TMenuItem;
    ODXML: TOpenDialog;
    PDetails: TPanel;
    SDXML: TSaveDialog;
    Splitter1: TSplitter;
    SBXML: TStatusBar;
    TBMain: TToolBar;
    TBNew: TToolButton;
    TBOPen: TToolButton;
    TBSave: TToolButton;
    ToolButton1: TToolButton;
    TVJSON: TTreeView;
    procedure ANewElementExecute(Sender: TObject);
    procedure ANewExecute(Sender: TObject);
    procedure AOpenExecute(Sender: TObject);
    procedure AQuitExecute(Sender: TObject);
    procedure ASaveAsExecute(Sender: TObject);
    procedure ASaveExecute(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure TVJSONSelectionChanged(Sender: TObject);
  private
    { private declarations }
    FJSON : TJSONObject;
    FFileName : String;
    procedure ClearNodeData;
    procedure DataTopLabel(ACaption: String);
    Function  NewElement(P : TJSONData;PT : TTreeNode) : TJSONData;
    procedure OpenFile(AFileName: String);
    procedure SaveToFile(AFileName: String);
    procedure SetFileName(AFileName: String);
    procedure ShowJSONValue(J: TJSONData);
    procedure ShowObjectData(O: TJSONObject);
    procedure ShowArrayData(A: TJSONArray);
    Function ShowNode(AParent: TTreeNode; AName : String; J: TJSONData) : TTreeNode;
    procedure ShowNodeData(J: TJSONData);
//    procedure ShowArrayData(J: TJSONData);
  public
    { public declarations }
    Procedure ShowObject(AJSON : TJSONObject);
  end;

var
  MainForm: TMainForm;

implementation

uses frmNewElement;

ResourceString
  SArrayData     = 'Array data:';
  SObjectData    = 'Object data:';
  SJSONValue     = 'Simple value:';
//  SNodeText      = 'Node text:';
//  SNodeData      = 'Attributes of node %s:';
  SElementName      = 'Name';
  SElementValue     = 'Value';
  SErrNoJSONObject = 'File does not contain a valid JSON object';
  SSelectNode    = 'Select a node to display its data here';
  SNewFile       = 'New XML document';
  SXMLFile       = 'XML document: %s';
  SErrNotaNode   = 'XPath expression result is not a DOM node';
  SErrSingleNode = 'XPath expression must return a single node';
  SGoto          = 'Goto node';
  SSpecifyPath   = 'Specify the path of the node you want to locate';
  SCurrentNode   = 'Current node path: %s';
  

{ TMainForm }

procedure TMainForm.SetFileName(AFileName : String);

begin
  If (AFileName='') then
    Caption:=SNewFile
  else
    Caption:=Format(SXMLFile,[AFileName]);
  FFileName:=AFileName;
end;

procedure TMainForm.AOpenExecute(Sender: TObject);
begin
  With ODXML do
    if Execute then
      OpenFile(FileName);
end;

procedure TMainForm.AQuitExecute(Sender: TObject);
begin
  Close;
end;

procedure TMainForm.ASaveAsExecute(Sender: TObject);
begin
  With SDXML do
    begin
    FileName:=FFileName;
    If Execute then
      SaveToFile(FileName);
    end;
end;

procedure TMainForm.SaveToFile(AFileName : String);

Var
  F : TextFile;

begin
  AssignFile(F,AFileName);
  Rewrite(F);
  Try
    Writeln(F,FJSON.AsJSon);
  Finally
    CloseFile(F);
  end;
  SetFileName(AFileName);
end;
  

procedure TMainForm.ASaveExecute(Sender: TObject);
begin
  If (FFileName<>'') then
    SaveToFile(FFileName)
  else
    ASaveAsExecute(Sender);
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  If (ParamCount>0) then
    if FileExists(ParamStr(1)) then
      OpenFile(ParamStr(1));
end;

procedure TMainForm.TVJSONSelectionChanged(Sender: TObject);
begin
  ClearNodeData;
  If Assigned(TVJSON.Selected) and Assigned(TVJSON.Selected.Data) then
    ShowNodeData(TJSONData(TVJSON.Selected.Data))
  else
    PDetails.Caption:=SSelectNode;
end;

procedure TMainForm.ClearNodeData;

begin
  With PDetails do
    While (ControlCount>0) do
      Controls[0].Free;
end;

procedure TMainForm.ShowNodeData(J : TJSONData);

begin
  PDetails.Caption:='';
  Case J.JSONType of
    jtArray  : ShowArrayData(J as TJSONArray);
    jtObject : ShowObjectData(J as TJSONObject);
  else
    ShowJSONValue(J)
  end;
end;

procedure TMainForm.DataTopLabel(ACaption : String);

Var
  L : TLabel;
  
begin
  L:=TLabel.Create(Self);
  L.Parent:=PDetails;
  L.Caption:=ACaption;
  L.AutoSize:=False;
  L.Align:=alTop;
  L.Alignment:=taCenter;
  L.Height:=24;
  L.Layout:=tlCenter;
end;

procedure TMainForm.ShowJSONValue(J : TJSONData);

Var
  M : TMemo;

begin
  DataTopLabel(SJSONValue);
  M:=TMemo.Create(Self);
  M.Parent:=PDetails;
  M.Align:=alClient;
  M.Lines.Text:=J.AsJSON;
end;

procedure TMainForm.ShowArrayData(A : TJSONArray);

Var
  G : TStringGrid;
  I : Integer;
  J : TJSONData;

begin
  DataTopLabel(SArrayData);
  G:=TStringGrid.Create(Self);
  G.Parent:=PDetails;
  G.Align:=alClient;
  G.FixedCols:=0;
  G.RowCount:=2;
  G.ColCount:=1;
  G.Cells[0,0]:=SElementValue;
  G.RowCount:=1+A.Count;
  G.Options:=G.Options+[goColSizing];
  If (G.RowCount>0) then
    G.FixedRows:=1
  else
    G.FixedRows:=0;
  For I:=1 to A.Count do
    begin
    J:=A.Items[i-1];
    If (J.JSONType in [jtArray,jtObject,jtNull]) then
      G.Cells[0,I]:=J.AsJSON
    else
      G.Cells[0,I]:=J.AsString;
    end;
end;

procedure TMainForm.ShowObjectData(O : TJSONObject);

Var
  L : TLabel;
  G : TStringGrid;
  I : Integer;
  J : TJSONData;
  
begin
  DataTopLabel(SObjectData);
  G:=TStringGrid.Create(Self);
  G.Parent:=PDetails;
  G.Align:=alClient;
  G.RowCount:=2;
  G.ColCount:=2;
  G.Cells[0,0]:=SElementName;
  G.ColWidths[0]:=120;
  G.Cells[1,0]:=SElementValue;
  G.RowCount:=1+O.Count;
  G.Options:=G.Options+[goColSizing];
  If (G.RowCount>0) then
    G.FixedRows:=1
  else
    G.FixedRows:=0;
  For I:=1 to O.Count do
    begin
    G.Cells[0,i]:=O.Names[i-1];
    J:=O.Items[i-1];
    If (J.JSONType in [jtArray,jtObject,jtNull]) then
      G.Cells[1,I]:=J.AsJSON
    else
      G.Cells[1,I]:=J.AsString;
    end;
end;

procedure TMainForm.ANewExecute(Sender: TObject);
begin
  FreeAndNil(FJSON);
  FJSON:=TJSONObject.Create;
  ShowObject(FJSON);
  SetFileName('');
end;


procedure TMainForm.OpenFile(AFileName : String);

Var
  J : TJSONData;
  F : TFileStream;

begin
  F:=TFileStream.Create(AFileName,fmOpenRead);
  Try
    With TJSONParser.Create(F) do
      try
        J:=Parse;
      Finally
        Free;
      end;
  Finally
    F.FRee;
  end;
  If J.JSONType<>jtObject then
    Raise Exception.Create(SErrNoJSONObject);
  If Assigned(FJSON) then
    FreeAndNil(FJSON);
  FJSON:=J as TJSONObject;
  SetFileName(AFileName);
  ShowObject(FJSON);
end;

Function TMainForm.ShowNode(AParent: TTreeNode; AName : String; J: TJSONData) : TTreeNode;

Var
  O : TJSONObject;
  I : Integer;

begin
  If Not (J.JSONType in [jtArray,jtObject]) then
    AName:=AName+' : '+J.AsJSON;
  Result:=TVJSON.Items.AddChild(AParent,AName);
  Result.Data:=J;
  If (J.Count<>0) then
    begin
    if (j is TJSONObject) then
      begin
      O:=J as TJSONObject;
      For I:=0 to O.Count-1 do
        ShowNode(Result,O.Names[i],O.Items[i]);
      end
    else if (j is TJSONArray) then
      begin
      For I:=0 to J.Count-1 do
        ShowNode(Result,IntToStr(I),J.Items[i]);
      end;
    end;
end;

procedure TMainForm.ShowObject(AJSON : TJSONObject);

begin
  With TVJSON.Items do
    begin
    BeginUpdate;
    Try
      Clear;
      If Assigned(AJSON) then
        begin
        ShowNode(Nil,'',AJSON);
        TVJSON.Selected:=TVJSON.Items.GetFirstNode;
        end;
    Finally
      EndUpdate;
    end;
    end;
end;

procedure TMainForm.ANewElementExecute(Sender: TObject);

Var
  P,N : TJSONData;
  PT : TTreeNode;

begin
  P:=Nil;
  PT:=TVJSON.Selected;
  If Assigned(PT) then
    P:=TJSONData(PT.Data);
  N:=NewElement(P,PT);
  If (N<>Nil) then
    begin
    ClearNodeData;
    ShowNodeData(N);
    end;
end;

Function TMainForm.NewElement(P : TJSONData;PT : TTreeNode) : TJSONData;

Var
  NT    : TJSonType;
  EN,EV : String;
  TN    : TTreeNode;
  I : Integer;

begin
  Result:=Nil;
  With TNewElementForm.Create(Self) do
    try
      NeedName:=P is TJSONObject;
      If (ShowModal<>mrOK) then
        Exit;
      NT:=DataType;
      EN:=ElementName;
      EV:=ElementValue;
    finally
      Free;
    end;
  Case NT of
    jtNumber : if TryStrToInt(EV,I) then
                 Result:=TJSONIntegerNumber.Create(i)
               else
                 Result:=TJSONFloatNumber.Create(StrToFloat(EV));
    jtNull : Result:=TJSONNull.Create;
    jtString : Result:=TJSONString.Create(EV);
    jtBoolean : Result:=TJSONBoolean.Create(StrToBool(EV));
    jtArray : Result:=TJSONArray.Create;
    jtObject : Result:=TJSONObject.Create;
  end;
  TN:=ShowNode(PT,EN,Result);
  TVJSON.Selected:=TN;
end;


initialization
  {$I frmmain.lrs}

end.

