unit frmmain;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, LResources, Forms, Controls, Graphics, Dialogs, ComCtrls,
  Menus, ActnList, ExtCtrls, DOM, XMLRead, XMLWrite, grids , StdCtrls, xpath;

type

  { TMainForm }

  TMainForm = class(TForm)
    AGoto: TAction;
    ANewElement: TAction;
    AQuit: TAction;
    ASaveAs: TAction;
    ASave: TAction;
    AOpen: TAction;
    ANew: TAction;
    ALMain: TActionList;
    ILMain: TImageList;
    MainMenu1: TMainMenu;
    Edit: TMenuItem;
    MIGoto: TMenuItem;
    MINewNode: TMenuItem;
    MISep1: TMenuItem;
    MIQuit: TMenuItem;
    MISaveAs: TMenuItem;
    MISave: TMenuItem;
    MIOpen: TMenuItem;
    MINew: TMenuItem;
    MFiLe: TMenuItem;
    ODXML: TOpenDialog;
    PDetails: TPanel;
    SDXML: TSaveDialog;
    Splitter1: TSplitter;
    SBXML: TStatusBar;
    TBMain: TToolBar;
    TBNew: TToolButton;
    TBOPen: TToolButton;
    TBSave: TToolButton;
    ToolButton1: TToolButton;
    TVXML: TTreeView;
    procedure AGotoExecute(Sender: TObject);
    procedure ANewElementExecute(Sender: TObject);
    procedure ANewExecute(Sender: TObject);
    procedure AOpenExecute(Sender: TObject);
    procedure AQuitExecute(Sender: TObject);
    procedure ASaveAsExecute(Sender: TObject);
    procedure ASaveExecute(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure TVXMLSelectionChanged(Sender: TObject);
  private
    { private declarations }
    FXML : TXMLDocument;
    FFileName : String;
    procedure ClearNodeData;
    procedure DataTopLabel(ACaption: String);
    procedure GotoNode(N: TDomNode);
    Function NewElement(P : TDomNode;PT : TTreeNode) : TDomNode;
    procedure OpenFile(AFileName: String);
    procedure SaveToFile(AFileName: String);
    procedure SetFileName(AFileName: String);
    procedure ShowElementData(E: TDomElement);
    procedure ShowNode(AParent: TTreeNode; E: TDomNode);
    procedure ShowNodeData(N: TDomNode);
    procedure ShowTextData(N: TDomNode);
  public
    { public declarations }
    Procedure ShowDocument(AXML : TXMLDocument);
  end;

var
  MainForm: TMainForm;

implementation

uses frmNewNode;

ResourceString
  SNodeText      = 'Node text:';
  SNodeData      = 'Attributes of node %s:';
  SAttrName      = 'Name';
  SAttrValue     = 'Value';
  SSelectNode    = 'Select a node to display its data here';
  SNewFile       = 'New XML document';
  SXMLFile       = 'XML document: %s';
  SErrNotaNode   = 'XPath expression result is not a DOM node';
  SErrSingleNode = 'XPath expression must return a single node';
  SGoto          = 'Goto node';
  SSpecifyPath   = 'Specify the path of the node you want to locate';
  SCurrentNode   = 'Current node path: %s';
  

{ TMainForm }

procedure TMainForm.SetFileName(AFileName : String);

begin
  If (AFileName='') then
    Caption:=SNewFile
  else
    Caption:=Format(SXMLFile,[AFileName]);
  FFileName:=AFileName;
end;

procedure TMainForm.AOpenExecute(Sender: TObject);
begin
  With ODXML do
    if Execute then
      OpenFile(FileName);
end;

procedure TMainForm.AQuitExecute(Sender: TObject);
begin
  Close;
end;

procedure TMainForm.ASaveAsExecute(Sender: TObject);
begin
  With SDXML do
    begin
    FileName:=FFileName;
    If Execute then
      SaveToFile(FileName);
    end;
end;

procedure TMainForm.SaveToFile(AFileName : String);

begin
  WriteXMLFile(FXML,AFileName);
  SetFileName(AFileName);
end;
  

procedure TMainForm.ASaveExecute(Sender: TObject);
begin
  If (FFileName<>'') then
    SaveToFile(FFileName)
  else
    ASaveAsExecute(Sender);
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  If (ParamCount>0) then
    if FileExists(ParamStr(1)) then
      OpenFile(ParamStr(1));
end;

procedure TMainForm.TVXMLSelectionChanged(Sender: TObject);
begin
  ClearNodeData;
  If Assigned(TVXML.Selected) and Assigned(TVXML.Selected.Data) then
    ShowNodeData(TDomNode(TVXML.Selected.Data))
  else
    PDetails.Caption:=SSelectNode;
end;

procedure TMainForm.ClearNodeData;

begin
  With PDetails do
    While (ControlCount>0) do
      Controls[0].Free;
end;

procedure TMainForm.ShowNodeData(N : TDomNode);

Var
  P : TDomNode;
  S : String;

begin
  PDetails.Caption:='';
  P:=N;
  S:='';
  While (P<>Nil) and Not (P is TXMLDocument) do
    begin
    If S<>'' then
      S:='/'+S;
    S:=P.NodeName+S;
    P:=P.ParentNode;
    end;
  SBXML.SimpleText:=Format(SCurrentNode,[S]);
  Case N.NodeType of
    TEXT_NODE    : ShowTextData(N as TDomText);
    ELEMENT_NODE : ShowElementData(N as TDomElement);
  end;
end;

procedure TMainForm.DataTopLabel(ACaption : String);

Var
  L : TLabel;
  
begin
  L:=TLabel.Create(Self);
  L.Parent:=PDetails;
  L.Caption:=ACaption;
  L.AutoSize:=False;
  L.Align:=alTop;
  L.Alignment:=taCenter;
  L.Height:=24;
  L.Layout:=tlCenter;
end;

procedure TMainForm.ShowTextData(N : TDomNode);

Var
  M : TMemo;

begin
  DataTopLabel(SNodeText);
  M:=TMemo.Create(Self);
  M.Parent:=PDetails;
  M.Lines.Text:=N.NodeValue;
  M.Align:=alClient;
end;

procedure TMainForm.ShowElementData(E : TDomElement);

Var
  L : TLabel;
  G : TStringGrid;
  I : Integer;
  N : TDomNode;
  
begin
  DataTopLabel(Format(SNodeData,[E.NodeName]));
  G:=TStringGrid.Create(Self);
  G.Parent:=PDetails;
  G.Align:=alClient;
  G.RowCount:=2;
  G.ColCount:=2;
  G.Cells[0,0]:=SAttrName;
  G.ColWidths[0]:=120;
  G.Cells[1,0]:=SAttrValue;
  G.RowCount:=1+E.Attributes.Length;
  G.Options:=G.Options+[goColSizing];
  If (G.RowCount>0) then
    G.FixedRows:=1
  else
    G.FixedRows:=0;
  For I:=1 to E.Attributes.Length do
    begin
    N:=E.Attributes[i-1];
    G.Cells[0,I]:=N.NodeName;
    G.Cells[1,I]:=N.NodeValue;
    end;

end;

procedure TMainForm.ANewExecute(Sender: TObject);
begin
  FreeAndNil(FXML);
  FXML:=TXMLDocument.Create;
  ShowDocument(FXML);
  SetFileName('');
end;


procedure TMainForm.OpenFile(AFileName : String);

Var
  ADoc : TXMLDocument;

begin
  ReadXMLFile(ADoc,AFileName);
  If Assigned(FXML) then
    FreeAndNil(FXML);
  FXML:=ADoc;
  SetFileName(AFileName);
  ShowDocument(FXML);
end;

procedure TMainForm.ShowNode(AParent: TTreeNode; E: TDomNode);

Var
  N : TTreeNode;
  D : TDomNode;
  
begin
  N:=TVXML.Items.AddChild(AParent,E.NodeName);
  N.Data:=E;
  D:=E.FirstChild;
  While (D<>Nil) do
    begin
    Case D.NodeType of
      ELEMENT_NODE : ShowNode(N,D);
      TEXT_NODE    : ShowNode(N,D);
    end;
    D:=D.NextSibling;
    end;
end;

procedure TMainForm.ShowDocument(AXML: TXMLDocument);

begin
  With TVXML.Items do
    begin
    BeginUpdate;
    Try
      Clear;
      If Assigned(AXML.DocumentElement) then
        begin
        ShowNode(Nil,AXML.DocumentElement);
        TVXML.Selected:=TVXML.Items.GetFirstNode;
        end;
    Finally
      EndUpdate;
    end;
    end;
end;

procedure TMainForm.ANewElementExecute(Sender: TObject);

Var
  P,N : TDomNode;
  PT : TTreeNode;

begin
  If (FXML=Nil) then
    begin
    FXML:=TXMLDocument.Create;
    ShowDocument(FXML);
    end;
  P:=FXML;
  PT:=TVXML.Selected;
  If Assigned(PT) then
    P:=TDOMNode(PT.Data);
  N:=NewElement(P,PT);
  If (N<>Nil) then
    begin
    ClearNodeData;
    ShowNodeData(N);
    end;
end;

procedure TMainForm.AGotoExecute(Sender: TObject);

Var
  S : String;
  V : TXPathVariable;
  N : TNodeSet;
  
begin
  If Assigned(FXML.DocumentElement) then
    S:='/'+FXML.DocumentElement.NodeName
  else
    S:='';
  If InputQuery(SGoto,SSpecifyPath,S) then
    begin
    V:=EvaluateXPathExpression(S,FXML);
    try
      If Not (V is TXPathNodeSetVariable) then
        ShowMessage(SErrNotaNode)
      else
        begin
        N:=V.AsNodeSet;
        If (N.Count<>1) then
          ShowMessage(SErrSingleNode)
        else
          GotoNode(TDOMNode(N.Items[0]));
        end;
    finally
      V.Free;
    end;
    end;
end;

procedure TMainForm.GotoNode(N : TDomNode);

Var
  TN : TTreeNode;
  
begin
  TN:=TVXML.Items.FindNodeWithData(N);
  If Assigned(TN) then
    begin
    TN.Selected:=True;
    TN.ExpandParents;
    end;
end;


Function TMainForm.NewElement(P : TDomNode;PT : TTreeNode) : TDomNode;

Var
  N,NN : TDomNode;
  NT : integer;
  NL : TNodeLocation;
  S : String;
  TN : TTreeNode;

begin
  Result:=Nil;
  With TNewNodeForm.Create(Self) do
    try
      HaveParent:=(P.ParentNode<>Nil);
      If (ShowModal<>mrOK) then
        Exit;
      NT:=NodeType;
      NL:=NodeLocation;
      S:=NodeText;
    finally
      Free;
    end;
  Case NT of
    ELEMENT_NODE : N:=FXML.CreateElement(S);
    TEXT_NODE : begin
                N:=FXML.CreateTextNode(S);
                NL:=nlLastChild;
                end;
  end;
  Case NL of
    nlFirstChild :
      begin
      NN:=P.FirstChild;
      If (NN<>Nil) then
        P.InsertBefore(N,NN)
      else
        P.AppendChild(N);
      TN:=TVXML.Items.AddChildFirst(PT,N.NodeName)
      end;
    nlLastChild  :
      begin
      P.AppendChild(N);
      TN:=TVXML.Items.AddChild(PT,N.NodeName);
      end;
    nlBeforeCurrent:
      begin
      P.ParentNode.InsertBefore(N,P);
      TN:=TVXML.Items.Insert(PT,N.NodeName);
      end;
    nlReplaceCurrent :
      begin
      P.ParentNode.ReplaceChild(N,P);
      PT.Text:=N.NodeName;
      TN:=PT;
      end;
  end;
  TN.Data:=N;
  TVXML.Selected:=TN;
  Result:=N;
end;


initialization
  {$I frmmain.lrs}

end.

